@@ -85,10 +85,10 @@ class Entry(object):
                 if item == attr.lower():
                     break
             else:
-                raise LDAPKeyError('key not found')
+                raise LDAPAttributeError('attribute not found')
             return self._attributes[attr]
 
-        raise LDAPKeyError('key must be a string')
+        raise LDAPAttributeError('attribute must be a string')
 
     def __setattr__(self, item, value):
         if item in self._attributes:
@@ -97,7 +97,16 @@ class Entry(object):
             raise LDAPEntryError('entry is read only')
 
     def __getitem__(self, item):
-        return self.__getattr__(item)
+        if isinstance(item, STRING_TYPES):
+            item = ''.join(item.split()).lower()
+            for attr in self._attributes:
+                if item == attr.lower():
+                    break
+            else:
+                raise LDAPKeyError('key not found')
+            return self._attributes[attr]
+
+        raise LDAPKeyError('key must be a string')
 
     def __eq__(self, other):
         if isinstance(other, Entry):
@@ -178,8 +187,13 @@ class Entry(object):
         if str == bytes:
             check_json_dict(json_entry)
 
-        json_output = json.dumps(json_entry, ensure_ascii=True, sort_keys=sort, indent=indent, check_circular=True,
-                                 default=format_json, separators=(',', ': '))
+        json_output = json.dumps(json_entry,
+                                 ensure_ascii=True,
+                                 sort_keys=sort,
+                                 indent=indent,
+                                 check_circular=True,
+                                 default=format_json,
+                                 separators=(',', ': '))
 
         if stream:
             stream.write(json_output)
