@@ -29,11 +29,11 @@ import json
 
 from .oid import CLASS_ABSTRACT, CLASS_STRUCTURAL, CLASS_AUXILIARY, ATTRIBUTE_USER_APPLICATION, \
     ATTRIBUTE_DIRECTORY_OPERATION, ATTRIBUTE_DISTRIBUTED_OPERATION, ATTRIBUTE_DSA_OPERATION
-from .. import CASE_INSENSITIVE_SCHEMA_NAMES, SEQUENCE_TYPES, STRING_TYPES
+from .. import SEQUENCE_TYPES, STRING_TYPES, get_config_parameter
 from ..utils.conv import escape_bytes, json_hook, check_json_dict, format_json, check_escape
 from ..utils.ciDict import CaseInsensitiveDict
 from ..protocol.formatters.standard import format_attribute_values
-from .oid import Oids, decode_oids, decode_syntax
+from .oid import Oids, decode_oids, decode_syntax, oid_to_string
 from ..core.exceptions import LDAPSchemaError, LDAPDefinitionError
 
 
@@ -120,7 +120,7 @@ class BaseServerInfo(object):
         if 'raw' not in definition or 'type' not in definition:
             raise LDAPDefinitionError('invalid JSON definition')
 
-        if CASE_INSENSITIVE_SCHEMA_NAMES:
+        if get_config_parameter('CASE_INSENSITIVE_SCHEMA_NAMES'):
             attributes = CaseInsensitiveDict()
         else:
             attributes = dict()
@@ -224,20 +224,21 @@ class DsaInfo(BaseServerInfo):
             r += ('  Alternative Servers: ' + str(self.alt_servers))
         r += linesep
         if isinstance(self.supported_controls, SEQUENCE_TYPES):
-            r += ('  Supported Controls: ' + linesep + linesep.join(['    ' + str(s) for s in self.supported_controls])) if self.supported_controls else ''
+            r += ('  Supported Controls: ' + linesep + linesep.join(['    ' + oid_to_string(s) for s in self.supported_controls])) if self.supported_controls else ''
         else:
             r += ('  Supported Controls: ' + str(self.supported_controls))
         r += linesep
         if isinstance(self.supported_extensions, SEQUENCE_TYPES):
-            r += ('  Supported Extensions: ' + linesep + linesep.join(['    ' + str(s) for s in self.supported_extensions])) if self.supported_extensions else ''
+            r += ('  Supported Extensions: ' + linesep + linesep.join(['    ' + oid_to_string(s) for s in self.supported_extensions])) if self.supported_extensions else ''
         else:
             r += ('  Supported Extensions: ' + str(self.supported_extensions))
         r += linesep
-        if isinstance(self.supported_features, SEQUENCE_TYPES):
-            r += ('  Supported Features: ' + linesep + linesep.join(['    ' + str(s) for s in self.supported_features])) if self.supported_features else ''
-        else:
-            r += ('  Supported Features: ' + str(self.supported_features))
-        r += linesep
+        if self.supported_features:
+            if isinstance(self.supported_features, SEQUENCE_TYPES):
+                r += ('  Supported Features: ' + linesep + linesep.join(['    ' + oid_to_string(s) for s in self.supported_features])) if self.supported_features else ''
+            else:
+                r += ('  Supported Features: ' + str(self.supported_features))
+            r += linesep
         if isinstance(self.supported_sasl_mechanisms, SEQUENCE_TYPES):
             r += ('  Supported SASL Mechanisms: ' + linesep + '    ' + ', '.join([str(s) for s in self.supported_sasl_mechanisms])) if self.supported_sasl_mechanisms else ''
         else:
@@ -248,6 +249,16 @@ class DsaInfo(BaseServerInfo):
         else:
             r += ('  Schema Entry: ' + str(self.schema_entry))
         r += linesep
+        if isinstance(self.vendor_name, SEQUENCE_TYPES) and len(self.vendor_name) == 1:
+            r += 'Vendor name: ' + self.vendor_name[0]
+        else:
+            r += 'Vendor name: ' + str(self.vendor_name)
+        r += linesep
+        if isinstance(self.vendor_version, SEQUENCE_TYPES) and len(self.vendor_version) == 1:
+            r += 'Vendor version: ' + self.vendor_version[0]
+        else:
+            r += 'Vendor version: ' + str(self.vendor_version)
+        r += linesep
         r += 'Other:' + linesep
         for k, v in self.other.items():
             r += '  ' + str(k) + ': ' + linesep
@@ -280,6 +291,18 @@ class SchemaInfo(BaseServerInfo):
         self.ldap_syntaxes = LdapSyntaxInfo.from_definition(attributes.pop('ldapSyntaxes', []))
         self.other = attributes  # remaining schema definition attributes not in RFC4512
 
+        # links attributes to objects
+        for object_class in self.object_classes:
+            for attribute in self.object_classes[object_class].must_contain:
+                try:
+                    self.attribute_types[attribute].mandatory_in.append(object_class)
+                except KeyError:
+                    pass
+            for attribute in self.object_classes[object_class].may_contain:
+                try:
+                    self.attribute_types[attribute].optional_in.append(object_class)
+                except KeyError:
+                    pass
     def __repr__(self):
         r = 'DSA Schema from: ' + self.schema_entry
         r += linesep
@@ -385,7 +408,7 @@ class BaseObjectInfo(object):
         if not definitions:
             return None
 
-        ret_dict = CaseInsensitiveDict() if CASE_INSENSITIVE_SCHEMA_NAMES else dict()
+        ret_dict = CaseInsensitiveDict() if get_config_parameter('CASE_INSENSITIVE_SCHEMA_NAMES') else dict()
         for object_definition in definitions:
             if [object_definition[0] == ')' and object_definition[:-1] == ')']:
                 if cls is MatchingRuleInfo:
@@ -583,8 +606,8 @@ class ObjectClassInfo(BaseObjectInfo):
                                 definition=definition)
         self.superior = superior
         self.kind = kind
-        self.must_contain = must_contain
-        self.may_contain = may_contain
+        self.must_contain = must_contain or []
+        self.may_contain = may_contain or []
 
     def __repr__(self):
         r = ''
@@ -636,6 +659,8 @@ class AttributeTypeInfo(BaseObjectInfo):
         self.collective = collective
         self.no_user_modification = no_user_modification
         self.usage = usage
+        self.mandatory_in = []
+        self.optional_in = []
 
     def __repr__(self):
         r = ''
@@ -648,6 +673,8 @@ class AttributeTypeInfo(BaseObjectInfo):
         r += (linesep + '  Substring rule: ' + list_to_string(self.substring)) if self.substring else ''
         r += (linesep + '  Syntax: ' + (self.syntax + (' [' + str(decode_syntax(self.syntax)))) + ']') if self.syntax else ''
         r += (linesep + '  Minimum Length: ' + str(self.min_length)) if isinstance(self.min_length, int) else ''
+        r += linesep + '  Mandatory in: ' + list_to_string(self.mandatory_in) if self.mandatory_in else ''
+        r += linesep + '  Optional in: ' + list_to_string(self.optional_in) if self.optional_in else ''
         return 'Attribute type' + BaseObjectInfo.__repr__(self).replace('<__desc__>', r)
 
 
