@@ -24,7 +24,7 @@ import unittest
 
 from ldap3 import ANONYMOUS, SASL, NTLM, Server, Connection, EXTERNAL, DIGEST_MD5
 from test import test_sasl_user, test_sasl_password, random_id, get_connection, drop_connection, test_sasl_realm, test_server_type, \
-    test_ntlm_user, test_ntlm_password, test_user, test_password
+    test_ntlm_user, test_ntlm_password, test_sasl_user_dn
 
 testcase_id = random_id()
 
@@ -39,15 +39,10 @@ class Test(unittest.TestCase):
         self.assertFalse(connection.bound)
 
     def test_bind_anonymous(self):
-        connection = get_connection(bind=False, lazy_connection=False, authentication=ANONYMOUS)
-        old_user = connection.user
-        connection.user = None
-        connection.open()
-        connection.bind()
+        connection = get_connection(bind=True, lazy_connection=False, authentication=ANONYMOUS)
         self.assertTrue(connection.bound)
         drop_connection(connection)
         self.assertFalse(connection.bound)
-        connection.user = old_user
 
     def test_bind_sasl_digest_md5(self):
         connection = get_connection(bind=False, authentication=SASL, sasl_mechanism=DIGEST_MD5, sasl_credentials=(test_sasl_realm, test_sasl_user, test_sasl_password, None))
@@ -56,11 +51,10 @@ class Test(unittest.TestCase):
         self.assertTrue(connection.bound)
         if test_server_type == 'EDIR':
             connected_user = connection.extend.novell.get_bind_dn()
-            for component in test_sasl_user.split('.'):
-                self.assertTrue(component in connected_user)
         else:
             connected_user = str(connection.extend.standard.who_am_i())
-            self.assertEqual(connected_user, test_sasl_user)
+
+        self.assertEqual(connected_user, test_sasl_user_dn)
         drop_connection(connection)
         self.assertFalse(connection.bound)
 
